# -*- coding: utf-8 -*-
"""
@author: Jose Manuel Martín Cabeza
"""

import StruPy2D as sp2

#---------------------------Portico a 2 aguas-------------------------------#

"""Nodos"""
p1 = sp2.Node(0,0)
p2 = sp2.Node(0,8)
p3 = sp2.Node(7.5,10)
p4 = sp2.Node(15,8)
p5 = sp2.Node(15,0)

"""Materiales y secciones"""
m1 = sp2.Material('acero',2.1e11,7850)
s1 = sp2.Section('sec1',A=20e-3,I=220e-6,h=0.4)

"""Barras"""
b1 = sp2.Bar(1, p1, p2, m1, s1)
b2 = sp2.Bar(2, p2, p3, m1, s1)
b3 = sp2.Bar(3, p3, p4, m1, s1)
b4 = sp2.Bar(4, p4, p5, m1, s1)

"""Cargas"""
q1 = sp2.Load('q1','DIS',fy = -1000)
q2 = sp2.Load('q2','DIS',fy = -1000)
b1.addLoad(q1)
b2.addLoad(q2)

"""Condiciones de contorno"""
p1.setBC(sp2.CLAMPED)
p5.setBC(sp2.CLAMPED)

"""Generar estructura"""
S = sp2.Structure([b1, b2, b3, b4])
S.show()
S.setWeight('off')

"""Calculo estatico"""
St = sp2.Static(S, run=True)
St.show(escaleDef=150)
for B in St.bars:
    print(B)

"""Calculo pandeo global"""
for i in range(3):
    Bu = sp2.Buckling(St, run=True, verModo=i+1)
    Bu.show()
print(Bu)

