# -*- coding: utf-8 -*-
"""
Created on Wed Aug 30 19:42:48 2017

@author:  Alvaro Magdaleno Gonzalez
"""
from StruPy2D.config import TOL

class Load:
    """Clase que representa una fuerza"""
    def __init__(self,ID,type,fx=0,fy=0,mz=0,a=0,b=0,ux1=0,uy1=0,th1=0,ux2=0,uy2=0,th2=0,T1=0,T2=0):
        
        if(type not in ["NOD","PUN","DIS","DEF","TER_T","TER_L"]):
            raise ValueError("Unknown type of load")
        
        self.type = type
        try:
            self.fx = float(fx)
            self.fy = float(fy)
            self.mz = float(mz)
            self.a = float(a)
            self.b = float(b)
            self.ux1 = float(ux1)
            self.uy1 = float(uy1)
            self.th1 = float(th1)
            self.ux2 = float(ux2)
            self.uy2 = float(uy2)
            self.th2 = float(th2)
            self.T1 = float(T1)
            self.T2 = float(T2)
            
        except ValueError as err:
            err.args = ('Expected numeric values in Load arguments; ' + err.args[0],)
            raise
            
        self.ID = ID
        self.bars = []
        self.nodes = []
        self.active = True
    
    def __str__(self):
        val = ''
        if(self.type == 'PUN'):
            type = 'Puntual load'
            if(abs(self.fx) > TOL):
                val = val + '    Fx = ' + str(self.fx)
            if(abs(self.fy) > TOL):
                if(abs(self.fx) > TOL):
                    val = val + '\n'
                val = val + '    Fy = ' + str(self.fy)
            if(abs(self.mz) > TOL):
                if(abs(self.fy) > TOL or abs(self.fx) > TOL):
                    val = val + '\n'
                val = val + '    Mz = ' + str(self.mz)
            val = val + '\n    a = ' + str(self.a)
            bars = 'Applied to bars: ' + str(self.bars)
            
        elif(self.type == 'DIS'):
            type = 'Distributed load'
            if(abs(self.fx) > TOL):
                val = val + '    qx = ' + str(self.fx)
            if(abs(self.fy) > TOL):
                if(abs(self.fx) > TOL):
                    val = val + '\n'
                val = val + '    qy = ' + str(self.fy)
            if(abs(self.mz) > TOL):
                if(abs(self.fy) > TOL or abs(self.fx) > TOL):
                    val = val + '\n'
                val = val + '    mz = ' + str(self.mz)
            val = val + '\n    a = ' + str(self.a) + '\n    b = ' + str(self.b)
            bars = 'Applied to bars: ' + str(self.bars)
            
        elif(self.type == 'DEF'):
            type = 'Previous deformation load'
            val = ''
            if(abs(self.ux1) > TOL):
                val = val + '    ux1 = ' + str(self.ux1) + '\n'
            if(abs(self.uy1) > TOL):
                val = val + '    uy1 = ' + str(self.uy1) + '\n'
            if(abs(self.th1) > TOL):
                val = val + '    th1 = ' + str(self.th1) + '\n'
            if(abs(self.ux2) > TOL):
                val = val + '    ux2 = ' + str(self.ux2) + '\n'
            if(abs(self.uy2) > TOL):
                val = val + '    uy2 = ' + str(self.uy2) + '\n'
            if(abs(self.th2) > TOL):
                val = val + '    th2 = ' + str(self.th2) + '\n'
            bars = 'Applied to bars: ' + str(self.bars)
            
        elif(self.type == 'TER_T'):
            type = 'Transversal thermic load'
            val = val + '    T1 = ' + str(self.T1) + '\n'
            val = val + '    T2 = ' + str(self.T2)
            bars = 'Applied to bars: ' + str(self.bars)
            
        elif(self.type == 'TER_L'):
            type = 'Longitudinal thermic load'
            val = val + '    T1 = ' + str(self.T1) + '\n'
            val = val + '    T2 = ' + str(self.T2)
            bars = 'Applied to bars: ' + str(self.bars)
            
        elif(self.type == 'NOD'):
            type = 'Nodal load'
            if(abs(self.fx) > TOL):
                val = val + '    Fx = ' + str(self.fx)
            if(abs(self.fy) > TOL):
                if(abs(self.fx) > TOL):
                    val = val + '\n'
                val = val + '    Fy = ' + str(self.fy)
            if(abs(self.mz) > TOL):
                if(abs(self.fy) > TOL or abs(self.fx) > TOL):
                    val = val + '\n'
                val = val + '    Mz = ' + str(self.mz)
            bars = 'Applied to nodes: ' + str(self.nodes)
        
        return type + '\n' + val + '\n' + bars
                
    def __repr__(self):
        return str(self.ID)
    
    def addBar(self,B): # Incovado internamente 
        if(type(B).__name__ == "Bar"):
            self.bars.append(B)
        elif(type(B).__name__ == "List"):
            self.bars += B
        else:
            print("Not a valid Bar object")
    
    def addNode(self,N): # Incovado internamente 
        if(type(N).__name__ == "Node"):
            self.nodes.append(N)
        elif(type(N).__name__ == "List"):
            self.nodes += N
        else:
            print("Not a valid Node object")
            
    def applyTo(self,obj):
        if(not type(obj).__name__ in ["Node","Bar","list"]):
            raise ValueError("Loads can only be applied to Nodes and Bars.")
            
        if(type(obj).__name__ == "Node"):
            if(self.type != "NOD"):
                raise ValueError("Nodal loads can only be applied to Nodes.")
            else:
                obj.addLoad(self)
                
        elif(type(obj).__name__ == "Bar"):
            if(self.type == "NOD"):
                raise ValueError("Bars do not accept nodal loads.")
            else:
                obj.addLoad(self)
        
        else:
            for O in obj:
                self.applyTo(O)
                
class LoadList(list):
    def __init__(self,loads=[]):
        if(type(loads).__name__ in ['Load','list','LoadList']):
            list.__init__(self,loads)
            
    def __repr__(self):
        return str([a for a in self])
    
    def ID(self):
        return [I.ID for I in self]
    
    def type(self):
        return [I.type for I in self]
    
    def has(self, load):
        if(type(load).__name__ == 'Load'):
            if(len(self) > 0):
                for L in self:
                    if(L.ID == load.ID):
                        return True
            else:
                return False
        else:
            print('Invalid type')
        
        return False
    
    def rem(self, ID):
        for L in self:
            if(L.ID == ID):
                self.remove(L)
                
class LoadGroup:
    def __init__(self, name, L=[], active=True):
        self.ID = str(name)
        self.loads = []
        self.active = True
        self.addLoad(L)
    
    def __str__(self):
        return 'ID:    '+self.ID+'\nLoads: '+str(self.loads)
        
    def __repr__(self):
        return 'ID:    '+self.ID+'\nLoads: '+str(self.loads)
    
    def __contains__(self, load):
        """Evalúa si la carga 'load' se encuentra en el grupo ya o no."""
        if(type(load).__name__ == 'Load'):
            load_list = [L.ID for L in self.loads]
            return load.ID in load_list
        
        elif(type(load).__name__ == 'str'):
            load_list = [L.ID for L in self.loads]
            return load in load_list
        
        else:
            raise TypeError('Expected Load or string')
        
    def rename(self, new_name):
        if(type(new_name).__name__ == 'str'):
            self.ID = new_name
        else:
            raise TypeError('Name must be a string')
        
    def addLoad(self,L):
        if(type(L).__name__ == "Load"):
            if(not L.ID in [lo.ID for lo in self.loads]):
                self.loads.append(L)
                L.active = self.active
        elif(type(L).__name__ == "list"):
            for lo in L:
                self.addLoad(lo)
        else:
            raise TypeError("LoadGroups can only contain Load objects")
    
    def delLoad(self,ID):
        if(type(ID).__name__ == 'str'):
            for L in self.loads:
                if(L.ID == ID):
                    self.loads.remove(L)
        else:
            raise TypeError('The ID must be a string.')
        
    def turn(self,stat):
        if(type(stat).__name__ != "str"):
            raise TypeError("LoadGroup state must be a string")
        if(not stat in ['ON','On','on','OFF','Off','off']):
            print("Input argument must be 'on' or 'off'")
        
        for L in self.loads:
            L.active = stat in ['ON','On','on']
            for B in L.bars:
                B.getFemp()