# -*- coding: utf-8 -*-
"""
Created on Wed Aug 30 19:42:56 2017

@author:  Alvaro Magdaleno Gonzalez
"""

class Material:
    def __init__(self,name,E=0.0,rho=0.0,al=0.0):
        self.ID = str(name)
        self.setProps(E,rho,al)
    
    def __str__(self):
        return 'ID: {}\r\n    E = {:.3e}, rho = {:.2f}, al = {:.3e}'.format(self.ID,self.E,self.rho,self.al)
    
    def __repr__(self):
        return 'ID: {}\r\n    E = {:.3e}, rho = {:.2f}, al = {:.3e}'.format(self.ID,self.E,self.rho,self.al)
        
    def setProps(self,E=0,rho=0,al=0):
        try:
            self.E = float(E)
            self.rho = float(rho)
            self.al = float(al)
        except TypeError as err:
            err.args = ('Expected numeric values for Material properties; ' + err.args[0],)
            raise
        
    def rename(self,name):
        self.ID = name
        
    def applyTo(self,b):
        """Aplica este material a la barra o barras indicadas (como lista)"""
        if(type(b).__name__ == "Bar"):
            b.mat = self
            b.getGlobal()
        elif(type(b).__name__ == "list"):
            for bar in b:
                if(type(bar).__name__ == 'Bar'):
                    bar.mat = self
                    bar.getGlobal()
                else:
                    raise TypeError("Materials can only be applied to Bar objects")
        else:
            raise TypeError("Materials can only be applied to Bar objects")

defMat = Material('default',2.1e11,7800,1.2e-5)

class MatList(list):
    def __init__(self,mats=[]):
        if(type(mats).__name__ in ['Material','list','MatList']):
            list.__init__(self,mats)
            
    def __repr__(self):
        return ',\n'.join([repr(a) for a in self])
    
    def __str__(self):
        return ',\n'.join([repr(a) for a in self])
    
    def E(self):
        return [I.E for I in self]
    
    def rho(self):
        return [I.rho for I in self]
    
    def al(self):
        return [I.al for I in self]
    
    def has(self, mat):
        if(type(mat).__name__ == 'Material'):
            if(len(self) > 0):
                for L in self:
                    if(L.E == mat.E and L.rho == mat.rho and L.al == mat.al and L.ID == mat.ID):
                        return True
            else:
                return False
        else:
            print('Invalid type')
        
        return False