# -*- coding: utf-8 -*-
"""
Created on Wed Sep 27 18:01:54 2017

@author:  Alvaro Magdaleno Gonzalez
"""

from math import atan, pi, sqrt, sin, cos
from StruPy2D.config import TOL
import matplotlib.pyplot as plt
import matplotlib.patches as patches

def drawLoad(ax,item,l):
    """Dibuja las cargas que hay en un elemento, punto o barra, en los axes 
    indicados en ax."""
    
    if(not type(item).__name__ in ["Node","Bar"]):
        print("Item not valid. Only points and bars.")
        return
    
    if(type(item).__name__ == "Node"):
        x,y = item.x, item.y
        props = {'head_length':0.06,
                 'head_width':0.03,
                 'length_includes_head':True,
                 'fc':'k',
                 'ec':'k'}
        if(len(item.loads) > 0):
            # Ángulos de las barras que confluyen, entre 0 y 2pi
            th_b = [0]*len(item.bars)
            for i in range(len(item.bars)):
                if(item == item.bars[i].p1):
                    th_b[i] = item.bars[i].th
                elif(item == item.bars[i].p2):
                    th_b[i]= item.bars[i].th - pi
                if(th_b[i] < 0):
                    th_b[i] = th_b[i] + 2*pi
            
            for L in item.loads:
                if(L.fx != 0 or L.fy != 0):
                    # Ángulo de la fuerza, entre 0 y 2pi grados
                    fx = L.fx
                    fy = L.fy
                    if(abs(fx) > TOL):
                        th = atan(fy/fx)
                        if(fx < 0):
                            th = th + pi
                        if(th < 0):
                            th = th + 2*pi
                    else:
                        if(fy >= 0):
                            th = pi/2
                        else:
                            th = 3*pi/2
                    
                    # Evaluación de las dos opciones: natural, de cola; si no, de punta
                    tail = [abs(x - th) for x in th_b]
                    for i in range(len(tail)):
                        if tail[i] > pi:
                            tail[i] = 2*pi - tail[i]
                            
                    head = [pi - x for x in tail]
                    
                    tail = sum(tail)
                    head = sum(head)
                    
                    # Se dibuja la flecha de cola (tail) o punta (head) en función
                    # de qué valor sea más alto
                    if (head >= tail):
                        p = patches.Arrow(item.x-l*cos(th),item.y-l*sin(th),
                                          l*cos(th),
                                          l*sin(th),
                                          width=0.33*l,
                                          facecolor='red',
                                          zorder = 4)
                    else:
                        p = patches.Arrow(item.x,item.y,
                                          l*cos(th),
                                          l*sin(th),
                                          width=0.33*l,
                                          facecolor='red',
                                          zorder = 4)
                    
                    ax.add_patch(p)
                    
                    # Valor neto de la carga
                    F = sqrt(fx**2 + fy**2)
                    
                if(L.mz != 0):
                    ax.plot(x, y, '+r')
                    if(L.mz > 0):
                        p = patches.FancyArrowPatch(
                                (x + 0.5*l, y - 0.866*l),
                                (x + 0.5*l, y + 0.866*l),
                                connectionstyle = 'arc3, rad=0.5',
                                mutation_scale = 25,
                                color = 'r',
                                lw = 0.1)
                    else:
                        p = patches.FancyArrowPatch(
                                (x + 0.5*l, y + 0.866*l),
                                (x + 0.5*l, y - 0.866*l),
                                connectionstyle = 'arc3, rad=-0.5',
                                mutation_scale = 25,
                                color = 'r',
                                lw = 0.1)
                    ax.add_patch(p)
    else:
        pass